module BigShift
  class BaseCommand
    class << self
      def execute(*args)
        self.new(*args).execute
      end
    end

    def execute
      @access_token = AccessTokenService.retrieve_token
      @response = on_execute
      scrub @response
      @response
    end

    private

    def connection
      Faraday.new(:url => base_url)
    end

    def post
      connection.post url, request_body, headers
    end

    def patch
      connection.patch url, request_body, headers
    end

    def url
      connection.build_url endpoint, params
    end

    def base_url
      'https://www.googleapis.com/bigquery/v2/projects/%s/datasets/%s' % [
        project_id,
        dataset_id,
      ]
    end

    def request_body
      body.to_json
    end

    def headers
      {
        'Content-Type' => 'application/json',
        'Authorization' => "Bearer #{@access_token}",
      }
    end

    def params
      {
        :key => @access_token,
      }
    end

    def project_id
      ENV['BIG_SHIFT_PROJECT_ID']
    end

    def dataset_id
      ENV['BIG_SHIFT_DATASET_ID']
    end

    def scrub(response)
      response.scrub @access_token, 'ACCESS_TOKEN'
      response.scrub project_id,   'PROJECT_ID'
      response.scrub dataset_id,   'DATASET_ID'
    end
  end
end
